﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using KHSoft.LocoTools;
using System.Globalization;
using System.ComponentModel;

namespace ThrottleWPF
{
    /// <summary>
    /// MainWindow.xaml の相互作用ロジック
    /// </summary>
    public partial class MainWindow : Window
    {
        private LocoToolsClient m_Client;
        // DataContextをLocoToolsSlotとして使用

        public MainWindow()
        {
            InitializeComponent();

            // LocoTools Clientを作成
            m_Client = new LocoToolsClient();
        }

        private void ConnectServer(object sender, RoutedEventArgs e)
        {
            // LocoTools Serverへ接続
            if (!m_Client.ConnectServer())
            {
                CheckBox1.IsChecked = false;
                MessageBox.Show("LocoTools Serverに接続できませんでした。");
                return;
            }

            // スロットを確保する
            LocoToolsSlot slot = m_Client.AllocateSlot(int.Parse(TextBox1.Text));
            if (slot == null)
            {
                // スロット確保に失敗
                CheckBox1.IsChecked = false;
                MessageBox.Show("スロットが確保できませんでした。");
                m_Client.Disconnect();
                return;
            }

            // DataContextにLocoToolsSlotオブジェクトを割り当てる
            DataContext = slot;

            // スライダのDataContextはスピード設定専用のクラスを別途設計
            Slider1.DataContext = new SpeedSetter();
            (Slider1.DataContext as SpeedSetter).slot = slot;
        }

        private void DisconnectServer(object sender, RoutedEventArgs e)
        {
            // 切断
            if (DataContext == null)
            {
                return;
            }

            (DataContext as LocoToolsSlot).Dispose();
            m_Client.Disconnect();
            DataContext = null;
            Slider1.DataContext = null;
        }

        private void DoReverse(object sender, RoutedEventArgs e)
        {
            if (DataContext == null)
            {
                return;
            }
            (DataContext as LocoToolsSlot).ReverseDirection();
        }

        private void ChangeSpeed(object sender, RoutedPropertyChangedEventArgs<double> e)
        {
            if (DataContext == null)
            {
                return;
            }

            int value = (int)(sender as Slider).Value;
            (DataContext as LocoToolsSlot).Speed = value;
        }
    }

    /// <summary>
    /// バインドしたオブジェクトが「nullでない」場合のチェックを行うヘルパクラス
    /// </summary>
    public class IsNullConverter : IValueConverter
    {
        public object Convert(object value, Type targetType, object parameter, CultureInfo culture)
        {
            return (value == null);
        }
        public object ConvertBack(object value, Type targetType, object parameter, CultureInfo culture)
        {
            throw new InvalidOperationException("IsNullConverter can only be used OneWay.");
        }
    }

    /// <summary>
    /// LocoToolsSlot.Speedを直接Slider.ValueにBindingすると、不必要なLocoToolsSlot.Speed.getが
    /// 大量に呼び出されてパフォーマンスダウンするため、プロパティセット専用のクラスを別途作る
    /// </summary>
    internal class SpeedSetter
    {
        internal LocoToolsSlot slot;

        public int Speed
        {
            set
            {
                slot.SetSpeed(value);
            }
        }
    }
}
